
# ------------------------------------------------------------------------
# Set of CMAKE macros used to manage the Java code in the Signal toolbox.
# Author: Loïc Besnard 
# Email: Loic.Besnard@irisa.fr
# ------------------------------------------------------------------------

# Generating the compiling of a JAVA program.
#  - FILENAME: source program to compile withtout suffix.
#  - INPUTDIR: the directory of FILENAME
#  - OUTPUTDIR: destination directory for class files
# ...
MACRO(JAVA_COMPILE INPUTDIR OUTPUTDIR FILENAME CLASSPATH WHERE ADDR_COMPILED_PACKAGE)
  SET(YY ${INPUTDIR}/${FILENAME}.java)
  FILE(TO_NATIVE_PATH ${YY} XX)
  ADD_CUSTOM_COMMAND(
    OUTPUT ${ADDR_COMPILED_PACKAGE}/${FILENAME}.class
    COMMAND ${CMAKE_COMMAND} -E echo "Compiling: ${INPUTDIR}/${FILENAME}"
    COMMAND ${CMAKE_COMMAND} -E make_directory ${OUTPUTDIR}
    COMMAND ${JAVA_COMPILE} ARGS ${XX} -classpath ${CLASSPATH} -d ${WHERE}
    COMMAND ${CMAKE_COMMAND} -E echo "Done."
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${YY}
    )
ENDMACRO(JAVA_COMPILE INPUTDIR OUTPUTDIR FILENAME CLASSPATH WHERE)

# Generating the compiling of a set of JAVA programs.
MACRO (PK_JAVA_COMPILING TARGET CP_CLASS_LIST MOD_LIST CLASS_PATH INPUTDIR OUTPUTDIR ADDR_COMPILED_PACKAGE)
  FOREACH(VAR ${MOD_LIST})
    JAVA_COMPILE(${INPUTDIR} ${OUTPUTDIR} "${VAR}" ${CLASS_PATH} ${OUTPUTDIR} ${ADDR_COMPILED_PACKAGE})
  ENDFOREACH(VAR)
  ADD_CUSTOM_TARGET(${TARGET} ALL DEPENDS ${CP_CLASS_LIST})
ENDMACRO (PK_JAVA_COMPILING TARGET CP_CLASS_LIST MOD_LIST CLASS_PATH INPUTDIR OUTPUTDIR)

# Generating the creating of a "jar" file (Java archive).
MACRO (PK_JAVA_MK_ARCHIVE TARGET ARCHIVE_NAME CP_CLASS_LIST CP_ADDR PACKAGE_ROOT)
ADD_CUSTOM_COMMAND(
    OUTPUT ${ARCHIVE_NAME}
    COMMAND ${CMAKE_COMMAND} -E echo_append "Building tar ... "
    COMMAND ${JAVA_ARCHIVE} cf ${ARCHIVE_NAME}  -C ${CP_ADDR} ${PACKAGE_ROOT}
    COMMAND ${CMAKE_COMMAND} -E echo "Done."
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${CP_CLASS_LIST}
    )
ADD_CUSTOM_TARGET(${TARGET} ALL DEPENDS ${ARCHIVE_NAME})
ENDMACRO (PK_JAVA_MK_ARCHIVE TARGET ARCHIVE_NAME CP_CLASS_LIST CP_ADDR  PACKAGE_ROOT)
