/* 
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __BUFFERMANAGER_H__
#define __BUFFERMANAGER_H__



                              /*============================*/
                              /* Author: A. GAMATIE         */
                              /* Last modified 03/13/2003   */
                              /*============================*/

/**************************************************************************************/
/*******************************   class BUFFER (headers) *****************************/
/**************************************************************************************/
/* The following class defines the buffer manager. So no guarantee is given that it   */
/* works correctly. However, it should not be very far from what you expect. :-)      */
/* Please, send any remark at the following address agamatie@irisa.fr                 */
/**************************************************************************************/





/******************************   BUFFER OBJECT : BEGIN ***************************/

/* Object representing APEX Buffer */
typedef class Buffer * ptBuffer;
class Buffer
{
   public :
   APEX_Buffer_type * buffer;
   Buffer(APEX_Buffer_type *b);
};

/*******************************   BUFFER OBJECT : END   **************************/




/**********************************************************************************/
/************************** BUFFER MANAGER_NODE  : BEGIN   ************************/
/**********************************************************************************/

/* Type of the contents of a manager node */
struct BuffAssoc_type{
  Comm_ComponentID_type BufferID;                      /* Identifier of a buffer. */  
  ptBuffer ptrBuffer;                           /* pointer on a buffer structure  */
};


/* Definition of the manager node */
typedef class BufferManagerNode *ptBufferManagerNode;
class BufferManagerNode
{
 private :
  BuffAssoc_type contents;                                 /* Contents of a node */
  ptBufferManagerNode next;                                         /* Next node */

 public :
  BufferManagerNode();
  BufferManagerNode(BuffAssoc_type componentRecord);
  ~BufferManagerNode();
  void setValue(BuffAssoc_type value);
  void setNext(ptBufferManagerNode ptvalue);
  BuffAssoc_type getValue();
  ptBufferManagerNode getNext();
};

/**********************************************************************************/
/**************************** BUFFER MANAGER_NODE  : END **************************/
/**********************************************************************************/



/**********************************************************************************/
/**************************** BUFFER  MANAGER : BEGIN *****************************/
/**********************************************************************************/

/* Manager type definition */
typedef class BufferManager *ptBufferManager;
class BufferManager
{
 private :
  ptBufferManagerNode first;                        /* First node in the manager */
  ptBufferManagerNode last;                          /* Last node in the manager */
  int max_size;                                     /* Size limit of the manager */
  int current_size;                    /* Current number of nodes in the manager */

  void setFirst(ptBufferManagerNode ptvalue);
  void setLast(ptBufferManagerNode ptvalue);
  ptBufferManagerNode getFirst();
  ptBufferManagerNode getLast();



 public :
  BufferManager();
  BufferManager(int size);
  ~BufferManager();


  bool BufferRecord(Comm_ComponentName_type buffer_Name, 
                    int buffer_Size, 
                    int message_Size, 
                    QueuingDiscipline_type queuing_Discipline, 
                    Comm_ComponentID_type& bufferID);
  /* If the manager was not full, a new buffer node is created, initialized
     with input parameters and it is returned true. Else, it is returned false. */


  bool BufferCheckName(Comm_ComponentName_type buffer_Name, Comm_ComponentID_type& bufferID); 
  /* If a buffer named buffer_Name exists in the manager, bufferID is its 
     identifier and it is returned true. Else, it is returned false */

 
  bool BufferCheckID(Comm_ComponentID_type bufferID, APEX_Buffer_type& buff);
  /* If there is an identifier bufferID in the manager, buff is the 
     buffer associated and it is returned true. Else, it is returned false */


  void BufferUpdate(APEX_Buffer_type buffer);
  /* The buffer from the manager, which has the same name as buff, is 
     updated */


  bool Full();
  /* Returns true if the manager size limit is reached. Else, it is returned
     false */


  void printFile(Comm_ComponentID_type bufferID, char *result_file);
  /* Prints bufferID status in result_file */
};


/*********************************************************************************/
/****************************** BUFFER MANAGER : END  ****************************/
/*********************************************************************************/


#endif // __BUFFERMANAGER_H__










