/* 
   Authors: Loïc Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/

/*	conversion functions	*/

#include "ansi.h"
#include <math.h>
#include "pK_GenCodeC_Predef.h"
#include "pK_Complex.h"
#include "pK_Conv.h"

/* ------------------result : logical --------------*/
EXTERN logical pK_pK_complex_2_logical(pK_complex E)
{
  return (logical)(((E.re==0.0)&&(E.im==0.0))?0:1);
}

EXTERN logical pK_pK_complexd_2_logical(pK_complexd E)
{
  return (logical)(((E.re==0.0)&&(E.im==0.0))?0:1);
}

/* ------------------result : short --------------*/

EXTERN short pK_float_2_short(float x)
{
  short y;
  if (x>=0.0) return (short)x;
  y = (short)x;
  return (short)(x == (float)y ? x : x-1);
}

EXTERN short pK_double_2_short(double x)
{
  short y;
  if (x>=0.0) return (short)x;
  y = (short)x;
  return (short)(x == (double)y ? x : x-1);
}

EXTERN short pK_pK_complex_2_short(pK_complex E)
{
  return pK_float_2_short(E.re);
}

EXTERN short pK_pK_complexd_2_short(pK_complexd E)
{
  return pK_double_2_short(E.re);
}

/* ------------------result : int --------------*/

EXTERN int pK_float_2_int(float x)
{
  int y;
  if (x>=0.0) return (int)x;
  y = (int)x;
  return ((int)(x == (float)y ? x : x-1));
}

EXTERN int pK_double_2_int(double x)
{
  int y;
  if (x>=0.0) return (int)x;
  y = (int)x;
  return ((int) (x == (double)y ? x : x-1));
}

EXTERN int pK_pK_complex_2_int(pK_complex E)
{
  return pK_float_2_int(E.re);
}

EXTERN int pK_pK_complexd_2_int(pK_complexd E)
{
  return pK_double_2_int(E.re);
}

/* ------------------result : long --------------*/
EXTERN long pK_double_2_long(double x)
{
  long y;
  if (x>=0.0) return (long)x;
  y = (long)x;
  return ((long)(x == (double)y ? x : x-1));
}

EXTERN long pK_float_2_long(float x)
{
  long y;
  if (x>=0.0) return (long)x;
  y = (long)x;
  return ((long)(x == (float)y ? x : x-1));
}

EXTERN long pK_pK_complex_2_long(pK_complex E)
{
  return pK_float_2_long(E.re);
}

EXTERN long pK_pK_complexd_2_long(pK_complexd E)
{
  return pK_double_2_long(E.re);
}

/* ------------------result : float --------------*/

EXTERN float pK_pK_complex_2_float(pK_complex E)
{
  return (float)(E.re);
}

EXTERN float pK_pK_complexd_2_float(pK_complexd E)
{
  return (float)(E.re);
}

/* ------------------result : double --------------*/
EXTERN double pK_pK_complex_2_double(pK_complex E)
{
  return (double)(E.re);
}

EXTERN double pK_pK_complexd_2_double(pK_complexd E)
{
  return (double)(E.re);
}

/* ------------------result : complex --------------*/
EXTERN pK_complex pK_logical_2_pK_complex(logical E)
{
  pK_complex S;
  S.re=(E?1.0:0.0);
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_short_2_pK_complex(short E)
{
  pK_complex S;
  S.re=(float)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_int_2_pK_complex(int E)
{
  pK_complex S;
  S.re=(float)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_long_2_pK_complex(long E)
{
  pK_complex S;
  S.re=(float)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_float_2_pK_complex(float E)
{
  pK_complex S;
  S.re=E;
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_double_2_pK_complex(double E)
{
  pK_complex S;
  S.re=(float)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complex pK_pK_complexd_2_pK_complex(pK_complexd E)
{
  pK_complex S;
  S.re=(float)(E.re);
  S.im=(float)(E.im);
  return S;
}


/* ------------------result : double complex --------------*/
EXTERN pK_complexd pK_logical_2_pK_complexd(logical E)
{
  pK_complexd S;
  S.re=(E?1.0:0.0);
  S.im=0.0;
  return S;
}


EXTERN pK_complexd pK_short_2_pK_complexd(short E)
{
  pK_complexd S;
  S.re=(double)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complexd pK_int_2_pK_complexd(int E)
{
  pK_complexd S;
  S.re=(double)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complexd pK_long_2_pK_complexd(long E)
{
  pK_complexd S;
  S.re=(double)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complexd pK_float_2_pK_complexd(float E)
{
  pK_complexd S;
  S.re=(double)E;
  S.im=0.0;
  return S;
}

EXTERN pK_complexd pK_double_2_pK_complexd(double E)
{
  pK_complexd S;
  S.re=E;
  S.im=0.0;
  return S;
}

EXTERN pK_complexd pK_pK_complex_2_pK_complexd(pK_complex E)
{
  pK_complexd S;
  S.re=(double)(E.re);
  S.im=(double)(E.im);
  return S;
}

/* ------------------result : char --------------*/

/* TG : Le codage ISO-Latin9, succdant je crois  ISO-Latin1,
   permet de coder un assez grand nombre de caractres
   (y compris "oe" colls ou le symbole euro, non inclus dans ISO-Latin1)
   sur 8 bits (comme ISO-Latin1) - le codage Ascii tait un codage sur 7 bits.
   Il existerait aussi un projet de norme Unicode sur 16 bits
   permettant de coder les caractres du monde entier (?)... */
EXTERN  char pK_short_2_char(short x)
{
  return ((char) x);
}
