module AADL_PROPERTY =

use INTERVALS;
use AADL_TYPE;

%an empty process is given for testing property Device_Dispatch_Protocol%
process Device_Dispatch_Protocol_property = 
{Supported_Dispatch_Protocols supp_dispatch_protocol;}()
   pragmas
       Comment "@parameter supp_dispatch_protocol: an enumeration value of Supported_Dispatch_Protocols"
     end pragmas
(||);

%an empty process is given for testing property Period%
process Period_property =
{Time_Units unit; i;}()
  pragmas
       Comment "@parameter unit: period unit"
               "@parameter i: period value"
     end pragmas
(||);

%an empty process is given for testing property Input_Time%
process Input_Time_property =
{Time_Units min_unit, max_unit; min_offset, max_offset;}
(? event Frozen_time_event,Reference_time_event;)
  pragmas
       Comment "@parameter min_unit: time unit of Input_Time low boundary"
               "@parameter max_unit: time unit of Input_Time high boundary"
               "@parameter min_offset: time value of Input_Time low boundary"
               "@parameter max_offset: time value of Input_Time high boundary"
               "@input Frozen_time_event: time event of frozen"
               "@input Reference_time_event: reference event"
     end pragmas
(||);

%an empty process is given for testing property Output_Time%
process Output_Time_property =
{Time_Units min_unit, max_unit; min_offset, max_offset;}
(? event Output_time_event,Reference_time_event;)
  pragmas
       Comment "@parameter min_unit: time unit of Output_Time low boundary"
               "@parameter max_unit: time unit of Output_Time high boundary"
               "@parameter min_offset: time value of Output_Time low boundary"
               "@parameter max_offset: time value of Output_Time high boundary"
               "@input Output_time_event: event of output time"
               "@input Reference_time_event: reference event"
     end pragmas
(||);

%an empty process is given for testing property Dispatch_Protocol%
process Dispatch_Protocol_property =
{Supported_Dispatch_Protocols supp_dispatch_protocol;}()
  pragmas
       Comment "@parameter supp_dispatch_protocol: an enumeration value of Supported_Dispatch_Protocols"
     end pragmas
(||);

%an empty process is given for testing property Deadline%
process Deadline_property =
{Time_Units unit; i;}()
  pragmas
       Comment "@parameter unit: deadline unit"
               "@parameter i: deadline value"
     end pragmas
(||);

%an empty process is given for testing property Actual_Processor_Binding%
process Actual_Processor_Binding_property = 
{doors_process1,cpiom1}()
  pragmas
       Comment "@parameter doors_process1: name of AADL process"
               "@parameter cpiom1: name of AADL processor"
     end pragmas
(||);

%an empty process is given for testing property Latency%
process Latency_property =
{Time_Units min_unit, max_unit; min_offset, max_offset;}
(? event Latency_time_event,Reference_time_event;)
  pragmas
       Comment "@parameter min_unit: time unit of Latency low boundary"
               "@parameter max_unit: time unit of Latency high boundary"
               "@parameter min_offset: time value of Latency low boundary"
               "@parameter max_offset: time value of Latency high boundary"
               "@input Latency_time_event: event of latency time"
               "@input Reference_time_event: reference event"
     end pragmas
(||);

%an empty process is given for testing property Time_Offset%
process Time_Offset_property =
{min_offset; Time_Units min_unit; max_offset; Time_Units max_unit;}
()
(||);

%an empty process is given for testing property Output_Time_Reference_Time%
process Output_Time_Reference_Time_property =
{Output_Time_Reference_Time ref_time;}
()
(||);

%an empty process is given for testing property Input_Time_Reference_Time%
process Input_Time_Reference_Time_property =
{Input_Time_Reference_Time ref_time;}
()
(||);

%an empty process is given for testing property Timing%
process Timing_property =
{Timing timing_value;}()
  pragmas
       Comment "@parameter timing_value: an enumeration value of Timing"
     end pragmas
(||);

%an empty process is given for testing property Fan_Out_Policy%
process Fan_Out_Policy_property =
{Fan_Out_Policy fan_out;}()
  pragmas
       Comment "@parameter fan_out: an enumeration value of Fan_Out_Policy"
     end pragmas
(||);

process Queue_Size_property =
{integer size;}
()
(||);

process Queue_Processing_Protocol_property =
{Queue_Processing_Protocol qpp;}
()
(||);

process Dequeue_Protocol_property =
{Dequeue_Protocol dp;}
()
(||);

process Dequeued_Items_property =
{integer k;}
()
(||);

process Compute_Entrypoint_property =
()
(||);

process Source_Text_property =
()
(||);

process Execution_Platform_property =
()
(||);

process Overflow_Handling_Protocol_property = 
{Overflow_Handling_Protocol pp;}
()
(||);

process Allowed_Processor_Binding_property =
()
(||);

process LAN_Config_Dictated_Processor_Binding_property =
()
(||);

process Clock_Period_property =
()
(||);

process Entity_Type_property =
()
(||);

process Long_Term_Rate_property =
()
(||);

process Burst_Rate_property =
()
(||);

process Network_Speed_property =
()
(||);

process Max_Transmission_Rate_property =
()
(||);

process Max_Packet_Size_property =
()
(||);

process Is_Partition_property =
()
(||);

process SporadicDispatch_constraint =
{integer n;}
(? event top; 
   event dispatch;
 ! boolean ok;)
(| cnt := top from dispatch
 | zcnt := cnt$ init n
 | ok := true when (zcnt >= n) when dispatch 
        default true when (zcnt = 0) when dispatch
        default false when dispatch
 |)
where
  integer cnt, zcnt;
end;

process PeriodicDispatch_constraint =
{integer n;}
(? event top; 
   event dispatch;
 ! boolean ok;)
(| cnt := top from dispatch
 | zcnt := cnt$ init n
 | ok := true when (zcnt = n) when dispatch 
        default true when (zcnt = 0) when dispatch
        default false when dispatch
 |)
where
  integer cnt, zcnt;
end;

process TimedDispatch_constraint =
{integer n;}
(? event top; 
   event dispatch;
 ! boolean ok;)
(| cnt := top from dispatch
 | zcnt := cnt$ init n
 | ok := true when (zcnt <= n) when dispatch 
        default false when dispatch
 |)
where
  integer cnt, zcnt;
end;

process HybridDispatch_constraint =
{integer n;}
(? event top; 
   event dispatch;
 ! boolean ok;)
(| cnt := top count n
 | zcnt := cnt$ init n
 | ok := true when (cnt = 0) when dispatch 
        default false when (cnt = 0)
        default true when dispatch
 |)
where
  integer cnt, zcnt, cc;
end;


end;
