module FIFO =

use SIGNALLIB;
use % AADL_MSGTYPE_boolean, AADL_FIFOTYPE, % AADL_TYPE;

process CREATE_FIFO = { integer fifo_size; msg_type default_msg; }
(? string fifo_name;
   
 ! ID_type fifo_ID; )
 pragmas
       Comment "create a FIFO"
     end pragmas
(| fifo_ID := FIFO_RECORD{fifo_size, default_msg}(fifo_name)
 |);


process FIFO_RECORD =  { integer fifo_size; msg_type default_msg; }
( ? string fifo_name;    
  ! ID_type fifo_ID;)
pragmas 
         C_Code "GLOBAL_FIFO_MANAGER->FIFORecord(&p1, &p2, &i1,  &o1)"
end pragmas;


process In_FIFO =
{integer k;}
(? ID_type fifo_ID; [k]msg_type x_in;)
 pragmas
       Comment "send x_in (k elements) to FIFO"
     end pragmas
(| fifo_ID ^= x_in
 | fifo := FIFO_CHECKID{}(fifo_ID)
 | fifo_new := SET_MESSAGE{k}(fifo_ID, fifo, x_in)
 |)
where
  FIFO_type fifo, fifo_new;
end;


process FIFO_CHECKID = 
( ? ID_type fifo_ID;
  ! FIFO_type fifo_out;)
pragmas 
         C_Code "GLOBAL_FIFO_MANAGER->FIFOCheckID(&i1,&o1)"
end pragmas;


process SET_MESSAGE =
{ integer k;}
(? ID_type fifo_ID; FIFO_type fifo; [k]msg_type x_in;
 ! FIFO_type fifo_new;)
 pragmas
       Comment "add [k]x_in at the end of fifo_ID "
               "@input fifo_ID: ID of FIFO"
               "@input fifo: FIFO"
               "@input [k]x_in: the elements to be added"
               "@output fifo_new: [k]x_in is added at the end of fifo. if k+fifo.nbmsg > fifo.size, the elements x_in[i] where i> fifo.size-fifo.nbmsg are ignored"
     end pragmas
(| fifo_new.name := fifo.name
 | fifo_new.size := fifo.size
 | fifo_new.nbmsg := (fifo.nbmsg + k) when ((fifo.nbmsg + k)<=fifo.size)
                     default fifo.size
 
| %array i to (MAX_QUEUE_SIZE-1) of
     fifo_new.fifo_queue[i] := if (i<fifo.nbmsg) then fifo.fifo_queue[i] 
                               else if ((i<fifo.size) and (i<(fifo.nbmsg+k))) then x_in[i-fifo.nbmsg]
                               else fifo.fifo_queue[i]
   end%  %array assignment problem%
   fifo_new.fifo_queue := [{i to (MAX_QUEUE_SIZE-1)}:fifo.fifo_queue[i]]
 
| FIFO_UPDATE(fifo_ID,fifo_new)
 |);



process FIFO_UPDATE =
(? ID_type fifo_ID; FIFO_type fifo;)
 pragmas
       Comment "update the fifo recored"
       C_Code "GLOBAL_FIFO_MANAGER->FIFOUpdate(&i1,&i2)"
end pragmas;



process Out_FIFO =
  {integer k; 
    msg_type default_msg;}
(? ID_type fifo_ID;
 ! [k]msg_type x_out; )
 pragmas
       Comment "get k elements (x_out) from FIFO, (suppose that k<fifo.nbmsg)"
               "@input fifo_ID: ID of FIFO"
               "@output [k]x_out: k elements of FIFO"
     end pragmas
(| fifo := FIFO_CHECKID{}(fifo_ID)
 | (fifo_new, x_out) := GET_MESSAGE{ k, default_msg}(fifo_ID, fifo)
 |)
where
  FIFO_type fifo, fifo_new;
end;



process GET_MESSAGE =
{integer k; 
 msg_type default_msg;}
(? ID_type fifo_ID; FIFO_type fifo;
 ! FIFO_type fifo_new; [k]msg_type x_out;)
 pragmas
       Comment "from fifo_ID, get k elements, return [k]x_out"
     end pragmas
(| fifo_new ^= x_out ^= fifo_ID ^= fifo
 | fifo_new.name := fifo.name
 | fifo_new.size := fifo.size
 | fifo_new.nbmsg := (fifo.nbmsg - k) when ((fifo.nbmsg - k)>0)
                     default 0

 | %array i to (MAX_QUEUE_SIZE-1) of
     fifo_new.fifo_queue[i] := if ((i<(fifo.nbmsg-k)) and (fifo.nbmsg>k)) then fifo.fifo_queue[k+i] 
                               else if ( (i>=(fifo.nbmsg-k)) and (fifo.nbmsg>k)) then default_msg
                               else default_msg                                                      
   end% %array assignment problem%
   fifo_new.fifo_queue := [{i to (MAX_QUEUE_SIZE-1)}:fifo.fifo_queue[i]]

 | %array i to k of
     x_out[i] := if (i < fifo.nbmsg) then fifo.fifo_queue[i]
                     else default_msg
                     end%
   x_out := [{i to k}:default_msg] when ^fifo

 | FIFO_UPDATE(fifo_ID, fifo_new)
 |);



process Move_k_FIFO =
{integer k; 
 msg_type default_msg;}
(? ID_type fifo_send, fifo_receive; event Move_time;
 ! [k]msg_type x_move;)
 pragmas
       Comment "at Move_time, move k elements from  fifo_send to fifo_receive"
     end pragmas
(| x_move := Out_FIFO{k, default_msg}(AT(fifo_send, Move_time))
 | In_FIFO{k}(AT(fifo_receive, Move_time),x_move)
 |);



process Move_all_FIFO =
{msg_type default_msg;}
(? ID_type fifo_send_ID, fifo_receive_ID; event Move_time;)
 pragmas
       Comment "at Move_time, move k elements from  fifo_send to fifo_receive"
     end pragmas
(| fifo_send := FIFO_CHECKID{}(AT(fifo_send_ID, Move_time))
 | fifo_receive := FIFO_CHECKID{}(AT(fifo_receive_ID, Move_time))

 | fifo_send_new ^= Move_time ^= fifo_send ^= fifo_receive ^= fifo_receive_new 
 | fifo_send_new.name := fifo_send.name
 | fifo_send_new.size := fifo_send.size
 | fifo_send_new.nbmsg := 0
 
 | %array i to (MAX_QUEUE_SIZE-1) of
     fifo_send_new.fifo_queue[i] := AT(default_msg, Move_time)
   end% %array assignment problem%
   fifo_send_new.fifo_queue := [{i to (MAX_QUEUE_SIZE-1)}: default_msg]

 | FIFO_UPDATE(AT(fifo_send_ID, Move_time), fifo_send_new)

 | fifo_receive_new.name := fifo_receive.name
 | fifo_receive_new.size := fifo_receive.size
 | fifo_receive_new.nbmsg := (fifo_receive.nbmsg + fifo_send.nbmsg) when ((fifo_receive.nbmsg + fifo_send.nbmsg)<fifo_receive.size)
                     default fifo_receive.size
 
| %array i to (MAX_QUEUE_SIZE-1) of
     fifo_receive_new.fifo_queue[i] := if (i<fifo_receive.nbmsg) then AT(fifo_receive.fifo_queue[i], Move_time)
                                                 else AT(fifo_send.fifo_queue[i-fifo_receive.nbmsg], Move_time)
                                                 end% %array assignment problem%
   fifo_receive_new.fifo_queue := [{i to (MAX_QUEUE_SIZE-1)}:fifo_receive.fifo_queue[i]]

 | FIFO_UPDATE(AT(fifo_receive_ID, Move_time), fifo_receive_new)
 |)
where
  FIFO_type fifo_send, fifo_receive, fifo_send_new, fifo_receive_new;
end;

process Get_Next =
{integer k;}
(? [k]msg_type array1;
   event get_time;
 ! msg_type next_value;)
(| cnt ^= next_value ^= zcnt ^= get_time
 | zcnt := cnt$1 init (-1)
 | cnt := zcnt when (zcnt=(k-1)) 
          default 0 when ((get_time from ^array1) =1)
          default zcnt+1
 | next_value := AT(array1[cnt], get_time)
 |)
where
  integer cnt, zcnt;
end;

end;
